<?php
/*
 * Пример вычисления коэффициента похожести текстов.
 * Сравниваем употребляемые в тексте слова. Порядок слов неважен.
 *
 * - производится очистка текста от BBCODE
 * - производится очистка от мусорных слов
 * - применяется стеммер Портера для вычисления русского корня слова
 * - строится упорядоченный уникальный массив слов
 * - два массива, склееные в строку, сравниваются функцией similar_text()
 *
 * (c) 2010, artoodetoo. Free for any use. No warranties.
 */

header('Content-Type: text/html; charset=utf-8');

mb_internal_encoding('utf-8');

if (get_magic_quotes_gpc())
	$_POST = array_map('stripslashes', $_POST);

$text1 = isset($_POST['text1']) ? $_POST['text1'] : '';
$text2 = isset($_POST['text2']) ? $_POST['text2'] : '';

//
// Слегка модифицированный класс "стеммер Портера для русского языка" (c) Дмитрий Котеров
//
class Stemmer
{
    private static
    	$_cache = array();

	const
		CACHING = TRUE,
    	VOWEL = '/аеиоуыэюя/u',
    	PERFECTIVEGROUND = '/((ив|ивши|ившись|ыв|ывши|ывшись)|((?<=[ая])(в|вши|вшись)))$/u',
    	REFLEXIVE = '/(с[яь])$/u',
    	ADJECTIVE = '/(ее|ие|ые|ое|ими|ыми|ей|ий|ый|ой|ем|им|ым|ом|его|ого|еых|ую|юю|ая|яя|ою|ею)$/u',
    	PARTICIPLE = '/((ивш|ывш|ующ)|((?<=[ая])(ем|нн|вш|ющ|щ)))$/u',
    	VERB = '/((ила|ыла|ена|ейте|уйте|ите|или|ыли|ей|уй|ил|ыл|им|ым|ены|ить|ыть|ишь|ую|ю)|((?<=[ая])(ла|на|ете|йте|ли|й|л|ем|н|ло|но|ет|ют|ны|ть|ешь|нно)))$/u',
    	NOUN = '/(а|ев|ов|ие|ье|е|иями|ями|ами|еи|ии|и|ией|ей|ой|ий|й|и|ы|ь|ию|ью|ю|ия|ья|я)$/u',
    	RVRE = '/^(.*?[аеиоуыэюя])(.*)$/u',
    	DERIVATIONAL = '/[^аеиоуыэюя][аеиоуыэюя]+[^аеиоуыэюя]+[аеиоуыэюя].*(?<=о)сть?$/u';

    private static function _s(&$s, $re, $to)
    {
        $orig = $s;
        $s = preg_replace($re, $to, $s);
        return $orig !== $s;
    }

    private static function _m($s, $re)
    {
        return preg_match($re, $s);
    }

    public static function stemWord($word)
    {
        //$word = mb_strtolower($word);
        $word = str_replace('ё', 'е', $word);

        # Check against cache of stemmed words
        if (self::CACHING && isset(self::$_cache[$word])) {
            return self::$_cache[$word];
        }

        $stem = $word;
        do {
          if (!preg_match(self::RVRE, $word, $p)) break;
          $start = $p[1];
          $rv = $p[2];
          if (!$rv) break;

          # Step 1
          if (!self::_s($rv, self::PERFECTIVEGROUND, '')) {
              self::_s($rv, self::REFLEXIVE, '');

              if (self::_s($rv, self::ADJECTIVE, '')) {
                  self::_s($rv, self::PARTICIPLE, '');
              } else {
                  if (!self::_s($rv, self::VERB, ''))
                      self::_s($rv, self::NOUN, '');
              }
          }

          # Step 2
          self::_s($rv, '/и$/', '');

          # Step 3
          if (self::_m($rv, self::DERIVATIONAL))
              self::_s($rv, '/ость?$/', '');

          # Step 4
          if (!self::_s($rv, '/ь$/', '')) {
              self::_s($rv, '/ейше?/', '');
              self::_s($rv, '/нн$/', 'н');
          }

          $stem = $start . $rv;
        } while(FALSE);

        if (self::CACHING) self::$_cache[$word] = $stem;
        return $stem;
    }

}

// Слегка модифицированная версия очистки из движка FluxBB
//
function splitWords($text)
{
	static $noise_match, $noise_replace, $stopwords;

	if (empty($noise_match))
	{
		$noise_match = 		array('[q', '[quote', '[code', '[url', '[img', '[email', '[color', '[colour', 'q]', 'quote]', 'code]', 'url]', 'img]', 'email]', 'color]', 'colour]', '^', '$', '&', '(', ')', '<', '>', '`', '\'', '"', '|', ',', '@', '_', '?', '%', '~', '+', '[', ']', '{', '}', ':', '\\', '/', '=', '#', ';', '!', '*');
		$noise_replace =	array('',   '',       '',      '',     '',     '',       '',       '',        '',   '',       '',      '',     '',     '',       '',       '',        ' ', ' ', ' ', ' ', ' ', ' ', ' ', '',  '',   ' ', ' ', ' ', ' ', '',  ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', '' ,  ' ', ' ', ' ', ' ', ' ', ' ');

		$stopwords = array('бля', 'кагбе', 'типа', 'сука', 'ну');
	}

	// Clean up
	$patterns[] = '#&[\#a-z0-9]+?;#i';
	$patterns[] = '#\b[\w]+:\/\/[a-z0-9\.\-]+(\/[a-z0-9\?\.%_\-\+=&\/~]+)?#';
	$patterns[] = '#\[\/?[a-z\*=\+\-]+(\:?[0-9a-z]+)?:[a-z0-9]{10,}(\:[a-z0-9]+)?=?.*?\]#';
	$text = preg_replace($patterns, ' ', ' ' . mb_strtolower($text) . ' ');

	// Filter out junk
	$text = str_replace($noise_match, $noise_replace, $text);

	// Strip out extra whitespace between words
	$text = trim(preg_replace('#\s+#', ' ', $text));

	// Fill an array with all the words
	$words = explode(' ', $text);

	if (!empty($words))
	{
		while (list($i, $word) = @each($words))
		{
			$words[$i] = Stemmer::stemWord(trim($word, '.'));
			$num_chars = mb_strlen($word);

			if (in_array($word, $stopwords))
				unset($words[$i]);
		}
	}

//	return $words;
	sort($words); return array_unique($words);
}

$words1 = implode(' ', splitWords($text1));
$words2 = implode(' ', splitWords($text2));
//$k = levenshtein($words1, $words2);
similar_text($words1, $words2, $pct); $k = (int)$pct;

?>
<form name="dbltext" action="<?php echo $_SERVER['PHP_SELF'] ?>" method="post">
	<table width="100%">
	<tr>
		<td><strong>Фрагмент - 1</strong><br>
		<textarea name="text1" rows=20 cols=60 wrap="on"><?=$text1?></textarea>
		</td>
		<td><strong>Фрагмент - 2</strong><br>
		<textarea name="text2" rows=20 cols=60 wrap="on"><?=$text2?></textarea>
		</td>
	</tr>
	<tr>
		<td><?php echo $words1 ?></td>
		<td><?php echo $words2 ?></td>
	</tr>
	<tr>
		<td colspan="2"><strong>Похожесть: <?php echo $k ?></strong></td>
	</tr>
	<tr>
		<td colspan="2"><input type="submit" value="Сравнить"></td>
	</tr>
	</table>
</form>